#!/usr/local/bin/perl

$readfilename = "sort.asm";
$writefilename = "sort.hex";

open (READFILE, "<$readfilename") || die ("can't open readfile");

while (<READFILE>) {
  my $instruction_type;
  my $rs_d, $rt_d, $rd_d, $funct, $imm7_d;
  my $line;
  my $binary;

  $line = $_;
  if ($line =~ /^$/) {
    next();
  }
  @linesplit = split (/\s+/,$line);

  #------------- bgn Assign Instruction Type ---------------# 
  $instruction_type = 'I';
  if (@linesplit[0] eq "lw")  {
    $op = "001";
  }
  elsif (@linesplit[0] eq "sw")  {
    $op = "010";
  }
  elsif (@linesplit[0] eq "addi")  {
    $op = "011";
  }
  elsif (@linesplit[0] eq "beq")  {
    $op = "100";
  }
  elsif (@linesplit[0] eq "j")  {
    $op = "101";
    $instruction_type = 'J';
  }
  elsif (@linesplit[0] eq "hlt")  {
    $op = "111";
    $instruction_type = 'J';
  }
  else {
    $op = "000";
    $instruction_type = 'R';
  }
  #------------- end Assign Instruction Type ---------------#
    
  if ($instruction_type eq 'R')  {   # <--begin R-type assignments

    #------------- bgn R-TYPE assign rs,rt,rd ---------------#
    @param1split1 = split (/r/,@linesplit[1]);
    @param1split2 = split (/,/,@param1split1[1]);
    $rd_d = @param1split2[0];

    @param2split1 = split (/r/,@linesplit[2]);
    if ((@linesplit[0] ne "inc") && (@linesplit[0] ne "dec") && 
       (@linesplit[0] ne "comp"))  { 
  
      @param2split2 = split (/,/,@param2split1[1]);    
      $rs_d = @param2split2[0];  

      @param3split1 = split (/r/,@linesplit[3]);
      $rt_d = @param3split1[1];
    }
    else  {
      $rs_d = @param2split1[1];
      $rt_d = '0';
    }
    #------------- end R-TYPE assign rs,rt,rd ---------------#
 
    #------------- bgn R-TYPE assign funct ------------------#
    if (@linesplit[0] eq "add")  {
      $funct = "0000"; }
    elsif (@linesplit[0] eq "sub")  {
      $funct = "0001"; }
    elsif (@linesplit[0] eq "inc")  {
      $funct = "0100"; }
    elsif (@linesplit[0] eq "dec")  {
      $funct = "0101"; }
    elsif (@linesplit[0] eq "sla")  {
      $funct = "1000"; }
    elsif (@linesplit[0] eq "sra")  {
      $funct = "1010"; }
    elsif (@linesplit[0] eq "and")  {
      $funct = "0010"; }
    elsif (@linesplit[0] eq "or")  {
      $funct = "0011"; }
    elsif (@linesplit[0] eq "comp")  {
      $funct = "0110"; }
    elsif (@linesplit[0] eq "sll")  {
      $funct = "1100"; }
    elsif (@linesplit[0] eq "srl")  {
      $funct = "1110"; }
    elsif (@linesplit[0] eq "slt")  {
      $funct = "1001"; }
    else  {
      print "\nERROR: ",@linesplit[0]," does not match any of the instructions!";
      exit();
    }
    #------------- end R-TYPE assign funct ------------------#
    
    $rs_short = unpack("B*", pack("n", $rs_d));
    $rs_b = substr($rs_short,13);
    $rt_short = unpack("B*", pack("n", $rt_d));
    $rt_b = substr($rt_short,13);
    $rd_short = unpack("B*", pack("n", $rd_d));
    $rd_b = substr($rd_short,13);
   
    $binary = $op.$rs_b.$rt_b.$rd_b.$funct;

    
  } #end if $instruction_type = 'R'    <-- end R-type assignments

  #------------- begin I-TYPE assignments ------------------#
  if ($instruction_type eq 'I')  {   
      
    @param1split1 = split (/r/,@linesplit[1]);
    @param1split2 = split (/,/,@param1split1[1]);
    $rt_d = @param1split2[0];
   
    if ((@linesplit[0] eq "lw") || (@linesplit[0] eq "sw"))  {    
      @param2split1 = split (/\(/,@linesplit[2]);
      $imm7_d = @param2split1[0];
      @param2split2 = split (/\)/,@param2split1[1]);
      @param2split3 = split (/r/,@param2split2[0]);
      $rs_d = @param2split3[1];  }
    else  {
      @param2split1 = split (/r/,@linesplit[2]);
      @param2split2 = split (/,/,@param2split1[1]);
      $rs_d = @param2split2[0];
      $imm7_d = @linesplit[3]; 
      if (@linesplit[0] eq "beq")  {  
        $rs_d = @param1split2[0];
        $rt_d = @param2split2[0];  }}

    $rs_short = unpack("B*", pack("n", $rs_d));
    $rs_b = substr($rs_short,13);
    $rt_short = unpack("B*", pack("n", $rt_d));
    $rt_b = substr($rt_short,13);
    $imm7_short = unpack("B*", pack("n", $imm7_d));
    $imm7_b = substr($imm7_short,9);
    
    $binary = $op.$rs_b.$rt_b.$imm7_b;
   
  } 
  #------------- end I-TYPE assignments ------------------#
 
  #------------- begin J-TYPE assignments ------------------#
  if ($instruction_type eq 'J')  {
    if (@linesplit[0] eq j)  {
      $target_d = @linesplit[1];
      $target_short = unpack("B*", pack("n", $target_d));
      $target_b = substr($target_short,3);  
    }
    else  {
      $target_b = "0000000000000";  }
  
    $binary = $op.$target_b;

  }  
  #------------- end J-TYPE assignments ------------------#

  $int = unpack("N", pack("B32", substr("0" x 32 . $binary, -32)));
  $decimal = sprintf("%d", $int);

  $hex_long = unpack("H*", pack("N", $decimal));
  $hex = substr($hex_long,4);
  $reversehex = reverse($hex);
  $temp = $temp.$reversehex;
   
}
close READFILE;

$temp = $temp." ";
open (WRITEFILE, ">$writefilename") || die ("can't open writefile");
print WRITEFILE $temp;
close WRITEFILE;